/**
	Haed a free program maker and generator
	Copyright (C) 2018  Florian Evaldsson

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <https://www.gnu.org/licenses/>.
*/
#include "includes.h"

/**
	Add a new object, eg Active object, backdrop ...
	
	@param tempobject INNER_REALLOC
		the top-level object-fond to add the object to
	@param name 
*/
void AddEksParentGUI(char *name,EksParent *holder,const char *type)
{
	EksParent *thisObject;
	
	b_tm("ADDING OBJECT %s",name);
	
	EksParent *objects=eks_parent_get_child_from_name(HAED_GLOBAL_INFORMATION.firstProgram->program,"objects");
	
	EksParent *activeObjects=eks_parent_get_child_from_name(objects,"active");
	
	thisObject=eks_parent_add_child(activeObjects,name,EKS_PARENT_TYPE_VALUE,NULL);
	
	eks_parent_custom_set(thisObject,holder,HAED_OBJECT_BUFFHOLDER);
	
	haed_trigghandler_add_object(thisObject);
	
	//Read the eks file for dimensions
	if(holder)
	{
		haed_object_set_imagebuff_from_bufholder(thisObject);

		haed_object_set_level_editor_icons(thisObject);
	}
	//active object s = constant
	if(type==ACTIVE_OBJECT_S)
	{
		haed_object_set_trigger_icons(thisObject);
	}
}

/**
	Adding trigger object
*/
void haed_object_add_trigger_std(char *name,EksParent *holder,const char *type)
{
	EksParent *thisObject;
	GdkPixbuf *iconbuff=NULL;
	
	b_tm("ADDING OBJECT %s",name);
	
	thisObject=eks_parent_new(name,EKS_PARENT_TYPE_VALUE,NULL,NULL);
	
	eks_parent_custom_set(thisObject,holder,HAED_OBJECT_BUFFHOLDER);
	
	haed_trigghandler_add_object(thisObject);

	GtkWidget *triggerMenuWidget,*triggerTopWidget;
	
	//only create the widgets to display it along triggers and to make a condition from it
	triggerMenuWidget=gtk_toggle_button_new();
	triggerTopWidget=gtk_toggle_button_new();
	
	if(type==TRIGGERS_SPECIAL_S)
	{
		iconbuff=gdk_pixbuf_new_from_file("./images/event_editor/special_conditions.png",NULL);
		g_signal_connect(triggerMenuWidget, "button_press_event",G_CALLBACK(trigger_conditions_special),(void*)thisObject);
	}
	else if(type==TRIGGERS_SOUND_S)
	{
		iconbuff=gdk_pixbuf_new_from_file("./images/event_editor/sound.png",NULL);
		g_signal_connect(triggerMenuWidget, "button_press_event",G_CALLBACK(trigger_conditions_sound),(void*)thisObject);
	}
	else if(type==TRIGGERS_STORY_S)
	{
		iconbuff=gdk_pixbuf_new_from_file("./images/event_editor/storyboard_controls.png",NULL);
		g_signal_connect(triggerMenuWidget, "button_press_event",G_CALLBACK(trigger_conditions_storyboard),(void*)thisObject);
	}
	else if(type==TRIGGERS_INPUT_S)
	{
		iconbuff=gdk_pixbuf_new_from_file("./images/event_editor/input.png",NULL);
		g_signal_connect(triggerMenuWidget, "button_press_event",G_CALLBACK(trigger_conditions_input),(void*)thisObject);
	}

	gtk_button_set_image(GTK_BUTTON(triggerMenuWidget),gtk_image_new_from_pixbuf(iconbuff));
	gtk_button_set_image(GTK_BUTTON(triggerTopWidget),gtk_image_new_from_pixbuf(iconbuff));
	
	gtk_box_pack_start(GTK_BOX(MAIN_ConditionsBox),triggerMenuWidget,0,0,0);

	int curpos=haed_triggerhandler_get_pos_from_object(thisObject);
	//for the top action objects
	gtk_grid_attach(GTK_GRID(HAED_GLOBAL_TRIGGER_GRID),triggerTopWidget,curpos+3,1,1,1);
	gtk_widget_show_all(triggerTopWidget);
}

void haed_object_set_level_editor_icons(EksParent *thisObject)
{
	GdkPixbuf *imagebuff=eks_parent_custom_get(thisObject,HAED_OBJECT_IMAGEBUFF);
	//set the icon
	GdkPixbuf *iconbuff=gdk_pixbuf_scale_simple(imagebuff,32,32,GDK_INTERP_NEAREST);
	
	//set the buttons images
	GtkWidget *drawMenuWidget=gtk_toggle_button_new();
	
	gtk_button_set_image(GTK_BUTTON(drawMenuWidget),gtk_image_new_from_pixbuf(iconbuff));
		g_signal_connect(drawMenuWidget, "clicked",G_CALLBACK(SetCurrentLayoutObjectPointer), NULL);
		g_signal_connect(drawMenuWidget, "button_press_event",G_CALLBACK(haed_level_editor_active_object_popup_menu),NULL);
		g_object_set_data(G_OBJECT(drawMenuWidget),"haed_object",(void*)(thisObject));
	gtk_box_pack_start(GTK_BOX(MAIN_LevelEditor_ListObjects),drawMenuWidget,0,0,0);
	gtk_widget_show_all(drawMenuWidget);
	
	eks_parent_custom_set(thisObject,drawMenuWidget,HAED_OBJECT_DRAW_MENU_WIDGET);
	
	g_object_unref(iconbuff);
}

void haed_object_set_trigger_icons(EksParent *thisObject)
{
	GdkPixbuf *imagebuff=eks_parent_custom_get(thisObject,HAED_OBJECT_IMAGEBUFF);
	//set the icon
	GdkPixbuf *iconbuff=gdk_pixbuf_scale_simple(imagebuff,32,32,GDK_INTERP_NEAREST);
	
	GtkWidget *triggerMenuWidget=gtk_toggle_button_new();
	GtkWidget *triggerButtonWidget=gtk_button_new();
	GtkWidget *triggerTopWidget=gtk_toggle_button_new();

	g_signal_connect(triggerButtonWidget,"clicked",G_CALLBACK(GetObjectFromSelection_ObjectSelect),(void*)thisObject);
	g_signal_connect(triggerMenuWidget, "button_press_event",G_CALLBACK(trigger_conditions_active_object),(void*)thisObject);

	gtk_button_set_image(GTK_BUTTON(triggerButtonWidget),gtk_image_new_from_pixbuf(iconbuff));
	gtk_button_set_image(GTK_BUTTON(triggerMenuWidget),gtk_image_new_from_pixbuf(iconbuff));
	gtk_button_set_image(GTK_BUTTON(triggerTopWidget),gtk_image_new_from_pixbuf(iconbuff));
	
	gtk_box_pack_start(GTK_BOX(MAIN_GetObjBox),triggerButtonWidget,0,0,0);
	
	gtk_box_pack_start(GTK_BOX(MAIN_ConditionsBox),triggerMenuWidget,0,0,0);

	int curpos=haed_triggerhandler_get_pos_from_object(thisObject);

	b_tm("curpos %d",curpos);
	//for the top action objects
	gtk_grid_attach(GTK_GRID(HAED_GLOBAL_TRIGGER_GRID),triggerTopWidget,curpos+3,1,1,1);
	gtk_widget_show_all(triggerTopWidget);
	
	eks_parent_custom_set(thisObject,triggerMenuWidget,HAED_OBJECT_TRIGGER_MENU_WIDGET);
	eks_parent_custom_set(thisObject,triggerButtonWidget,HAED_OBJECT_TRIGGER_BUTTON_WIDGET);
	eks_parent_custom_set(thisObject,triggerTopWidget,HAED_OBJECT_TRIGGER_TOP_WIDGET);
	g_object_unref(iconbuff);
}

void haed_object_set_imagebuff_from_bufholder(EksParent *thisObject)
{
	EksParent *holder=eks_parent_custom_get(thisObject,HAED_OBJECT_BUFFHOLDER);

	GdkPixbuf *testPixbuf=haed_pixbufholder_get_pixbuf_from_name(holder,GLOBAL_ANIMATION_STOPPED);
	
	EksParent *bufSettings=eks_parent_get_child(eks_parent_get_child_from_name(eks_parent_get_child_from_name(eks_parent_get_child_from_name(holder,GLOBAL_ANIMATION_STOPPED),"ALL_MIRROR"),"FRAMES"),0);

	int framex=eks_parent_get_int(eks_parent_get_child(bufSettings,0));
	int framey=eks_parent_get_int(eks_parent_get_child(bufSettings,1));
	int framewidth=eks_parent_get_int(eks_parent_get_child(bufSettings,2));
	int frameheight=eks_parent_get_int(eks_parent_get_child(bufSettings,3));
	//int framehs=eks_parent_get_int(eks_parent_get_child(bufSettings,4));
	//int framehs=eks_parent_get_int(eks_parent_get_child(bufSettings,5));
	
	b_tm("x %d,y %d,w %d,h %d\n",framex,framey,framewidth,frameheight);

	//get only the corner for the icon
	GdkPixbuf *imagebuff=gdk_pixbuf_new(GDK_COLORSPACE_RGB,1,8,framewidth,frameheight);
	gdk_pixbuf_copy_area(testPixbuf,framex,framey,framewidth,frameheight,imagebuff,0,0);
	eks_parent_custom_set(thisObject,imagebuff,HAED_OBJECT_IMAGEBUFF);
}


char *haed_object_get_string(EksParent *object, const char *assignparam)
{
	return haed_get_string_from_pos(object,assignparam,0);
}

void haed_object_set_string(EksParent *object, const char *assignparam,const char *setparam, int delete)
{
	haed_set_string_from_pos(object,assignparam,setparam,delete,0);
}

intptr_t haed_object_get_int(EksParent *object, const char *assignparam)
{
	return haed_get_int_from_pos(object,assignparam,0);
}

void haed_object_set_int(EksParent *object, const char *assignparam,int setparam, int delete)
{
	haed_set_int_from_pos(object,assignparam,setparam,delete,0);
}

double haed_object_get_double(EksParent *object, const char *assignparam)
{
	return haed_get_double_from_pos(object,assignparam,0);
}

inline void haed_object_set_double(EksParent *object, const char *assignparam,double setparam, int delete)
{
	haed_set_double_from_pos(object,assignparam,setparam,delete,0);
}

/**
	Initiates the left-side bar in the level editor
	
	
	-
*/
void haed_object_list_visual_init(void)
{
	MAIN_LevelEditor_ListObjects=gtk_box_new(GTK_ORIENTATION_VERTICAL,2);
						
		//This is just definitions for the objects shown in the left row
		MAIN_LevelEditor_ListObjectsIconsImage[0]=gtk_image_new_from_file("./images/level_editor/active_objects.png");
		gtk_box_pack_start(GTK_BOX(MAIN_LevelEditor_ListObjects),MAIN_LevelEditor_ListObjectsIconsImage[0],0,0,0);

	gtk_box_pack_start(GTK_BOX(HAED_GLOBAL_LIST_OBJECTS),MAIN_LevelEditor_ListObjects,0,1,0);
	
	gtk_widget_show_all(HAED_GLOBAL_LIST_OBJECTS);
}

/**
	resets the left-side bar in the level editor
	
	-
*/
void haed_object_list_visual_reset(void)
{
	if(MAIN_LevelEditor_ListObjects)
		gtk_widget_destroy(MAIN_LevelEditor_ListObjects);
	
	haed_object_list_visual_init();
}
